import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 13
key = xcffib.ExtensionKey("RECORD")
_events = {}
_errors = {}


class Range8(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.first, self.last = unpacker.unpack("=BB")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=BB", self.first, self.last))
        return buf.getvalue()

    fixed_size = 2

    @classmethod
    def synthetic(cls, first, last):
        self = cls.__new__(cls)
        self.first = first
        self.last = last
        return self


class Range16(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.first, self.last = unpacker.unpack("=HH")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=HH", self.first, self.last))
        return buf.getvalue()

    fixed_size = 4

    @classmethod
    def synthetic(cls, first, last):
        self = cls.__new__(cls)
        self.first = first
        self.last = last
        return self


class ExtRange(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.major = Range8(unpacker)
        unpacker.pad(Range16)
        self.minor = Range16(unpacker)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            self.major.pack()
            if hasattr(self.major, "pack")
            else Range8.synthetic(*self.major).pack()
        )
        buf.write(
            self.minor.pack()
            if hasattr(self.minor, "pack")
            else Range16.synthetic(*self.minor).pack()
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, major, minor):
        self = cls.__new__(cls)
        self.major = major
        self.minor = minor
        return self


class Range(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.core_requests = Range8(unpacker)
        unpacker.pad(Range8)
        self.core_replies = Range8(unpacker)
        unpacker.pad(ExtRange)
        self.ext_requests = ExtRange(unpacker)
        unpacker.pad(ExtRange)
        self.ext_replies = ExtRange(unpacker)
        unpacker.pad(Range8)
        self.delivered_events = Range8(unpacker)
        unpacker.pad(Range8)
        self.device_events = Range8(unpacker)
        unpacker.pad(Range8)
        self.errors = Range8(unpacker)
        self.client_started, self.client_died = unpacker.unpack("=BB")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            self.core_requests.pack()
            if hasattr(self.core_requests, "pack")
            else Range8.synthetic(*self.core_requests).pack()
        )
        buf.write(
            self.core_replies.pack()
            if hasattr(self.core_replies, "pack")
            else Range8.synthetic(*self.core_replies).pack()
        )
        buf.write(
            self.ext_requests.pack()
            if hasattr(self.ext_requests, "pack")
            else ExtRange.synthetic(*self.ext_requests).pack()
        )
        buf.write(
            self.ext_replies.pack()
            if hasattr(self.ext_replies, "pack")
            else ExtRange.synthetic(*self.ext_replies).pack()
        )
        buf.write(
            self.delivered_events.pack()
            if hasattr(self.delivered_events, "pack")
            else Range8.synthetic(*self.delivered_events).pack()
        )
        buf.write(
            self.device_events.pack()
            if hasattr(self.device_events, "pack")
            else Range8.synthetic(*self.device_events).pack()
        )
        buf.write(
            self.errors.pack()
            if hasattr(self.errors, "pack")
            else Range8.synthetic(*self.errors).pack()
        )
        buf.write(struct.pack("=B", self.client_started))
        buf.write(struct.pack("=B", self.client_died))
        return buf.getvalue()

    @classmethod
    def synthetic(
        cls,
        core_requests,
        core_replies,
        ext_requests,
        ext_replies,
        delivered_events,
        device_events,
        errors,
        client_started,
        client_died,
    ):
        self = cls.__new__(cls)
        self.core_requests = core_requests
        self.core_replies = core_replies
        self.ext_requests = ext_requests
        self.ext_replies = ext_replies
        self.delivered_events = delivered_events
        self.device_events = device_events
        self.errors = errors
        self.client_started = client_started
        self.client_died = client_died
        return self


class HType:
    FromServerTime = 1 << 0
    FromClientTime = 1 << 1
    FromClientSequence = 1 << 2


class CS:
    CurrentClients = 1
    FutureClients = 2
    AllClients = 3


class ClientInfo(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.client_resource, self.num_ranges = unpacker.unpack("=II")
        self.ranges = xcffib.List(unpacker, Range, self.num_ranges)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=II", self.client_resource, self.num_ranges))
        buf.write(xcffib.pack_list(self.ranges, Range))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, client_resource, num_ranges, ranges):
        self = cls.__new__(cls)
        self.client_resource = client_resource
        self.num_ranges = num_ranges
        self.ranges = ranges
        return self


class BadContextError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        (self.invalid_record,) = unpacker.unpack("=xx2xI")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(struct.pack("=x2xI", self.invalid_record))
        return buf.getvalue()


BadBadContext = BadContextError
_errors[0] = BadContextError


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.major_version, self.minor_version = unpacker.unpack("=xx2x4xHH")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class GetContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.enabled, self.element_header, self.num_intercepted_clients = (
            unpacker.unpack("=xB2x4xB3xI16x")
        )
        self.intercepted_clients = xcffib.List(
            unpacker, ClientInfo, self.num_intercepted_clients
        )
        self.bufsize = unpacker.offset - base


class GetContextCookie(xcffib.Cookie):
    reply_type = GetContextReply


class EnableContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.category,
            self.element_header,
            self.client_swapped,
            self.xid_base,
            self.server_time,
            self.rec_sequence_num,
        ) = unpacker.unpack("=xB2x4xBB2xIII8x")
        self.data = xcffib.List(unpacker, "B", self.length * 4)
        self.bufsize = unpacker.offset - base


class EnableContextCookie(xcffib.Cookie):
    reply_type = EnableContextReply


class recordExtension(xcffib.Extension):
    def QueryVersion(self, major_version, minor_version, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", major_version, minor_version))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self, major_version, minor_version):
        return self.QueryVersion(major_version, minor_version, is_checked=True)

    def QueryVersionUnchecked(self, major_version, minor_version):
        return self.QueryVersion(major_version, minor_version, is_checked=False)

    def CreateContext(
        self,
        context,
        element_header,
        num_client_specs,
        num_ranges,
        client_specs,
        ranges,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIB3xII", context, element_header, num_client_specs, num_ranges
            )
        )
        buf.write(xcffib.pack_list(client_specs, "I"))
        buf.write(xcffib.pack_list(ranges, Range))
        return self.send_request(1, buf, is_checked=is_checked)

    def CreateContextChecked(
        self,
        context,
        element_header,
        num_client_specs,
        num_ranges,
        client_specs,
        ranges,
    ):
        return self.CreateContext(
            context,
            element_header,
            num_client_specs,
            num_ranges,
            client_specs,
            ranges,
            is_checked=True,
        )

    def CreateContextUnchecked(
        self,
        context,
        element_header,
        num_client_specs,
        num_ranges,
        client_specs,
        ranges,
    ):
        return self.CreateContext(
            context,
            element_header,
            num_client_specs,
            num_ranges,
            client_specs,
            ranges,
            is_checked=False,
        )

    def RegisterClients(
        self,
        context,
        element_header,
        num_client_specs,
        num_ranges,
        client_specs,
        ranges,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIB3xII", context, element_header, num_client_specs, num_ranges
            )
        )
        buf.write(xcffib.pack_list(client_specs, "I"))
        buf.write(xcffib.pack_list(ranges, Range))
        return self.send_request(2, buf, is_checked=is_checked)

    def RegisterClientsChecked(
        self,
        context,
        element_header,
        num_client_specs,
        num_ranges,
        client_specs,
        ranges,
    ):
        return self.RegisterClients(
            context,
            element_header,
            num_client_specs,
            num_ranges,
            client_specs,
            ranges,
            is_checked=True,
        )

    def RegisterClientsUnchecked(
        self,
        context,
        element_header,
        num_client_specs,
        num_ranges,
        client_specs,
        ranges,
    ):
        return self.RegisterClients(
            context,
            element_header,
            num_client_specs,
            num_ranges,
            client_specs,
            ranges,
            is_checked=False,
        )

    def UnregisterClients(
        self, context, num_client_specs, client_specs, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", context, num_client_specs))
        buf.write(xcffib.pack_list(client_specs, "I"))
        return self.send_request(3, buf, is_checked=is_checked)

    def UnregisterClientsChecked(self, context, num_client_specs, client_specs):
        return self.UnregisterClients(
            context, num_client_specs, client_specs, is_checked=True
        )

    def UnregisterClientsUnchecked(self, context, num_client_specs, client_specs):
        return self.UnregisterClients(
            context, num_client_specs, client_specs, is_checked=False
        )

    def GetContext(self, context, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context))
        return self.send_request(4, buf, GetContextCookie, is_checked=is_checked)

    def GetContextChecked(self, context):
        return self.GetContext(context, is_checked=True)

    def GetContextUnchecked(self, context):
        return self.GetContext(context, is_checked=False)

    def EnableContext(self, context, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context))
        return self.send_request(5, buf, EnableContextCookie, is_checked=is_checked)

    def EnableContextChecked(self, context):
        return self.EnableContext(context, is_checked=True)

    def EnableContextUnchecked(self, context):
        return self.EnableContext(context, is_checked=False)

    def DisableContext(self, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context))
        return self.send_request(6, buf, is_checked=is_checked)

    def DisableContextChecked(self, context):
        return self.DisableContext(context, is_checked=True)

    def DisableContextUnchecked(self, context):
        return self.DisableContext(context, is_checked=False)

    def FreeContext(self, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context))
        return self.send_request(7, buf, is_checked=is_checked)

    def FreeContextChecked(self, context):
        return self.FreeContext(context, is_checked=True)

    def FreeContextUnchecked(self, context):
        return self.FreeContext(context, is_checked=False)


xcffib._add_ext(key, recordExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 1
key = xcffib.ExtensionKey("SHAPE")
_events = {}
_errors = {}
from . import xproto


class SO:
    Set = 0
    Union = 1
    Intersect = 2
    Subtract = 3
    Invert = 4


class SK:
    Bounding = 0
    Clip = 1
    Input = 2


class NotifyEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.shape_kind,
            self.affected_window,
            self.extents_x,
            self.extents_y,
            self.extents_width,
            self.extents_height,
            self.server_time,
            self.shaped,
        ) = unpacker.unpack("=xB2xIhhHHIB11x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(
            struct.pack(
                "=B2xIhhHHIB11x",
                self.shape_kind,
                self.affected_window,
                self.extents_x,
                self.extents_y,
                self.extents_width,
                self.extents_height,
                self.server_time,
                self.shaped,
            )
        )
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(
        cls,
        shape_kind,
        affected_window,
        extents_x,
        extents_y,
        extents_width,
        extents_height,
        server_time,
        shaped,
    ):
        self = cls.__new__(cls)
        self.shape_kind = shape_kind
        self.affected_window = affected_window
        self.extents_x = extents_x
        self.extents_y = extents_y
        self.extents_width = extents_width
        self.extents_height = extents_height
        self.server_time = server_time
        self.shaped = shaped
        return self


_events[0] = NotifyEvent


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.major_version, self.minor_version = unpacker.unpack("=xx2x4xHH")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class QueryExtentsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.bounding_shaped,
            self.clip_shaped,
            self.bounding_shape_extents_x,
            self.bounding_shape_extents_y,
            self.bounding_shape_extents_width,
            self.bounding_shape_extents_height,
            self.clip_shape_extents_x,
            self.clip_shape_extents_y,
            self.clip_shape_extents_width,
            self.clip_shape_extents_height,
        ) = unpacker.unpack("=xx2x4xBB2xhhHHhhHH")
        self.bufsize = unpacker.offset - base


class QueryExtentsCookie(xcffib.Cookie):
    reply_type = QueryExtentsReply


class InputSelectedReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.enabled,) = unpacker.unpack("=xB2x4x")
        self.bufsize = unpacker.offset - base


class InputSelectedCookie(xcffib.Cookie):
    reply_type = InputSelectedReply


class GetRectanglesReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.ordering, self.rectangles_len = unpacker.unpack("=xB2x4xI20x")
        self.rectangles = xcffib.List(unpacker, xproto.RECTANGLE, self.rectangles_len)
        self.bufsize = unpacker.offset - base


class GetRectanglesCookie(xcffib.Cookie):
    reply_type = GetRectanglesReply


class shapeExtension(xcffib.Extension):
    def QueryVersion(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self):
        return self.QueryVersion(is_checked=True)

    def QueryVersionUnchecked(self):
        return self.QueryVersion(is_checked=False)

    def Rectangles(
        self,
        operation,
        destination_kind,
        ordering,
        destination_window,
        x_offset,
        y_offset,
        rectangles_len,
        rectangles,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xBBBxIhh",
                operation,
                destination_kind,
                ordering,
                destination_window,
                x_offset,
                y_offset,
            )
        )
        buf.write(xcffib.pack_list(rectangles, xproto.RECTANGLE))
        return self.send_request(1, buf, is_checked=is_checked)

    def RectanglesChecked(
        self,
        operation,
        destination_kind,
        ordering,
        destination_window,
        x_offset,
        y_offset,
        rectangles_len,
        rectangles,
    ):
        return self.Rectangles(
            operation,
            destination_kind,
            ordering,
            destination_window,
            x_offset,
            y_offset,
            rectangles_len,
            rectangles,
            is_checked=True,
        )

    def RectanglesUnchecked(
        self,
        operation,
        destination_kind,
        ordering,
        destination_window,
        x_offset,
        y_offset,
        rectangles_len,
        rectangles,
    ):
        return self.Rectangles(
            operation,
            destination_kind,
            ordering,
            destination_window,
            x_offset,
            y_offset,
            rectangles_len,
            rectangles,
            is_checked=False,
        )

    def Mask(
        self,
        operation,
        destination_kind,
        destination_window,
        x_offset,
        y_offset,
        source_bitmap,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xBB2xIhhI",
                operation,
                destination_kind,
                destination_window,
                x_offset,
                y_offset,
                source_bitmap,
            )
        )
        return self.send_request(2, buf, is_checked=is_checked)

    def MaskChecked(
        self,
        operation,
        destination_kind,
        destination_window,
        x_offset,
        y_offset,
        source_bitmap,
    ):
        return self.Mask(
            operation,
            destination_kind,
            destination_window,
            x_offset,
            y_offset,
            source_bitmap,
            is_checked=True,
        )

    def MaskUnchecked(
        self,
        operation,
        destination_kind,
        destination_window,
        x_offset,
        y_offset,
        source_bitmap,
    ):
        return self.Mask(
            operation,
            destination_kind,
            destination_window,
            x_offset,
            y_offset,
            source_bitmap,
            is_checked=False,
        )

    def Combine(
        self,
        operation,
        destination_kind,
        source_kind,
        destination_window,
        x_offset,
        y_offset,
        source_window,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xBBBxIhhI",
                operation,
                destination_kind,
                source_kind,
                destination_window,
                x_offset,
                y_offset,
                source_window,
            )
        )
        return self.send_request(3, buf, is_checked=is_checked)

    def CombineChecked(
        self,
        operation,
        destination_kind,
        source_kind,
        destination_window,
        x_offset,
        y_offset,
        source_window,
    ):
        return self.Combine(
            operation,
            destination_kind,
            source_kind,
            destination_window,
            x_offset,
            y_offset,
            source_window,
            is_checked=True,
        )

    def CombineUnchecked(
        self,
        operation,
        destination_kind,
        source_kind,
        destination_window,
        x_offset,
        y_offset,
        source_window,
    ):
        return self.Combine(
            operation,
            destination_kind,
            source_kind,
            destination_window,
            x_offset,
            y_offset,
            source_window,
            is_checked=False,
        )

    def Offset(
        self, destination_kind, destination_window, x_offset, y_offset, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xB3xIhh", destination_kind, destination_window, x_offset, y_offset
            )
        )
        return self.send_request(4, buf, is_checked=is_checked)

    def OffsetChecked(self, destination_kind, destination_window, x_offset, y_offset):
        return self.Offset(
            destination_kind, destination_window, x_offset, y_offset, is_checked=True
        )

    def OffsetUnchecked(self, destination_kind, destination_window, x_offset, y_offset):
        return self.Offset(
            destination_kind, destination_window, x_offset, y_offset, is_checked=False
        )

    def QueryExtents(self, destination_window, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", destination_window))
        return self.send_request(5, buf, QueryExtentsCookie, is_checked=is_checked)

    def QueryExtentsChecked(self, destination_window):
        return self.QueryExtents(destination_window, is_checked=True)

    def QueryExtentsUnchecked(self, destination_window):
        return self.QueryExtents(destination_window, is_checked=False)

    def SelectInput(self, destination_window, enable, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIB3x", destination_window, enable))
        return self.send_request(6, buf, is_checked=is_checked)

    def SelectInputChecked(self, destination_window, enable):
        return self.SelectInput(destination_window, enable, is_checked=True)

    def SelectInputUnchecked(self, destination_window, enable):
        return self.SelectInput(destination_window, enable, is_checked=False)

    def InputSelected(self, destination_window, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", destination_window))
        return self.send_request(7, buf, InputSelectedCookie, is_checked=is_checked)

    def InputSelectedChecked(self, destination_window):
        return self.InputSelected(destination_window, is_checked=True)

    def InputSelectedUnchecked(self, destination_window):
        return self.InputSelected(destination_window, is_checked=False)

    def GetRectangles(self, window, source_kind, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIB3x", window, source_kind))
        return self.send_request(8, buf, GetRectanglesCookie, is_checked=is_checked)

    def GetRectanglesChecked(self, window, source_kind):
        return self.GetRectangles(window, source_kind, is_checked=True)

    def GetRectanglesUnchecked(self, window, source_kind):
        return self.GetRectangles(window, source_kind, is_checked=False)


xcffib._add_ext(key, shapeExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 0
MINOR_VERSION = 11
key = xcffib.ExtensionKey("RENDER")
_events = {}
_errors = {}
from . import xproto


class PictType:
    Indexed = 0
    Direct = 1


class Picture:
    _None = 0


class PictOp:
    Clear = 0
    Src = 1
    Dst = 2
    Over = 3
    OverReverse = 4
    In = 5
    InReverse = 6
    Out = 7
    OutReverse = 8
    Atop = 9
    AtopReverse = 10
    Xor = 11
    Add = 12
    Saturate = 13
    DisjointClear = 16
    DisjointSrc = 17
    DisjointDst = 18
    DisjointOver = 19
    DisjointOverReverse = 20
    DisjointIn = 21
    DisjointInReverse = 22
    DisjointOut = 23
    DisjointOutReverse = 24
    DisjointAtop = 25
    DisjointAtopReverse = 26
    DisjointXor = 27
    ConjointClear = 32
    ConjointSrc = 33
    ConjointDst = 34
    ConjointOver = 35
    ConjointOverReverse = 36
    ConjointIn = 37
    ConjointInReverse = 38
    ConjointOut = 39
    ConjointOutReverse = 40
    ConjointAtop = 41
    ConjointAtopReverse = 42
    ConjointXor = 43
    Multiply = 48
    Screen = 49
    Overlay = 50
    Darken = 51
    Lighten = 52
    ColorDodge = 53
    ColorBurn = 54
    HardLight = 55
    SoftLight = 56
    Difference = 57
    Exclusion = 58
    HSLHue = 59
    HSLSaturation = 60
    HSLColor = 61
    HSLLuminosity = 62


class PolyEdge:
    Sharp = 0
    Smooth = 1


class PolyMode:
    Precise = 0
    Imprecise = 1


class CP:
    Repeat = 1 << 0
    AlphaMap = 1 << 1
    AlphaXOrigin = 1 << 2
    AlphaYOrigin = 1 << 3
    ClipXOrigin = 1 << 4
    ClipYOrigin = 1 << 5
    ClipMask = 1 << 6
    GraphicsExposure = 1 << 7
    SubwindowMode = 1 << 8
    PolyEdge = 1 << 9
    PolyMode = 1 << 10
    Dither = 1 << 11
    ComponentAlpha = 1 << 12


class SubPixel:
    Unknown = 0
    HorizontalRGB = 1
    HorizontalBGR = 2
    VerticalRGB = 3
    VerticalBGR = 4
    _None = 5


class Repeat:
    _None = 0
    Normal = 1
    Pad = 2
    Reflect = 3


class PictFormatError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadPictFormat = PictFormatError
_errors[0] = PictFormatError


class PictureError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 1))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadPicture = PictureError
_errors[1] = PictureError


class PictOpError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 2))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadPictOp = PictOpError
_errors[2] = PictOpError


class GlyphSetError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 3))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadGlyphSet = GlyphSetError
_errors[3] = GlyphSetError


class GlyphError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 4))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadGlyph = GlyphError
_errors[4] = GlyphError


class DIRECTFORMAT(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.red_shift,
            self.red_mask,
            self.green_shift,
            self.green_mask,
            self.blue_shift,
            self.blue_mask,
            self.alpha_shift,
            self.alpha_mask,
        ) = unpacker.unpack("=HHHHHHHH")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=HHHHHHHH",
                self.red_shift,
                self.red_mask,
                self.green_shift,
                self.green_mask,
                self.blue_shift,
                self.blue_mask,
                self.alpha_shift,
                self.alpha_mask,
            )
        )
        return buf.getvalue()

    fixed_size = 16

    @classmethod
    def synthetic(
        cls,
        red_shift,
        red_mask,
        green_shift,
        green_mask,
        blue_shift,
        blue_mask,
        alpha_shift,
        alpha_mask,
    ):
        self = cls.__new__(cls)
        self.red_shift = red_shift
        self.red_mask = red_mask
        self.green_shift = green_shift
        self.green_mask = green_mask
        self.blue_shift = blue_shift
        self.blue_mask = blue_mask
        self.alpha_shift = alpha_shift
        self.alpha_mask = alpha_mask
        return self


class PICTFORMINFO(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.id, self.type, self.depth = unpacker.unpack("=IBB2x")
        self.direct = DIRECTFORMAT(unpacker)
        (self.colormap,) = unpacker.unpack("=I")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=IBB2x", self.id, self.type, self.depth))
        buf.write(
            self.direct.pack()
            if hasattr(self.direct, "pack")
            else DIRECTFORMAT.synthetic(*self.direct).pack()
        )
        buf.write(struct.pack("=I", self.colormap))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, id, type, depth, direct, colormap):
        self = cls.__new__(cls)
        self.id = id
        self.type = type
        self.depth = depth
        self.direct = direct
        self.colormap = colormap
        return self


class PICTVISUAL(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.visual, self.format = unpacker.unpack("=II")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=II", self.visual, self.format))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, visual, format):
        self = cls.__new__(cls)
        self.visual = visual
        self.format = format
        return self


class PICTDEPTH(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.depth, self.num_visuals = unpacker.unpack("=BxH4x")
        self.visuals = xcffib.List(unpacker, PICTVISUAL, self.num_visuals)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=BxH4x", self.depth, self.num_visuals))
        buf.write(xcffib.pack_list(self.visuals, PICTVISUAL))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, depth, num_visuals, visuals):
        self = cls.__new__(cls)
        self.depth = depth
        self.num_visuals = num_visuals
        self.visuals = visuals
        return self


class PICTSCREEN(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.num_depths, self.fallback = unpacker.unpack("=II")
        self.depths = xcffib.List(unpacker, PICTDEPTH, self.num_depths)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=II", self.num_depths, self.fallback))
        buf.write(xcffib.pack_list(self.depths, PICTDEPTH))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, num_depths, fallback, depths):
        self = cls.__new__(cls)
        self.num_depths = num_depths
        self.fallback = fallback
        self.depths = depths
        return self


class INDEXVALUE(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.pixel, self.red, self.green, self.blue, self.alpha = unpacker.unpack(
            "=IHHHH"
        )
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=IHHHH", self.pixel, self.red, self.green, self.blue, self.alpha
            )
        )
        return buf.getvalue()

    fixed_size = 12

    @classmethod
    def synthetic(cls, pixel, red, green, blue, alpha):
        self = cls.__new__(cls)
        self.pixel = pixel
        self.red = red
        self.green = green
        self.blue = blue
        self.alpha = alpha
        return self


class COLOR(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.red, self.green, self.blue, self.alpha = unpacker.unpack("=HHHH")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=HHHH", self.red, self.green, self.blue, self.alpha))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, red, green, blue, alpha):
        self = cls.__new__(cls)
        self.red = red
        self.green = green
        self.blue = blue
        self.alpha = alpha
        return self


class POINTFIX(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.x, self.y = unpacker.unpack("=ii")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=ii", self.x, self.y))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, x, y):
        self = cls.__new__(cls)
        self.x = x
        self.y = y
        return self


class LINEFIX(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.p1 = POINTFIX(unpacker)
        unpacker.pad(POINTFIX)
        self.p2 = POINTFIX(unpacker)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            self.p1.pack()
            if hasattr(self.p1, "pack")
            else POINTFIX.synthetic(*self.p1).pack()
        )
        buf.write(
            self.p2.pack()
            if hasattr(self.p2, "pack")
            else POINTFIX.synthetic(*self.p2).pack()
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, p1, p2):
        self = cls.__new__(cls)
        self.p1 = p1
        self.p2 = p2
        return self


class TRIANGLE(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.p1 = POINTFIX(unpacker)
        unpacker.pad(POINTFIX)
        self.p2 = POINTFIX(unpacker)
        unpacker.pad(POINTFIX)
        self.p3 = POINTFIX(unpacker)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            self.p1.pack()
            if hasattr(self.p1, "pack")
            else POINTFIX.synthetic(*self.p1).pack()
        )
        buf.write(
            self.p2.pack()
            if hasattr(self.p2, "pack")
            else POINTFIX.synthetic(*self.p2).pack()
        )
        buf.write(
            self.p3.pack()
            if hasattr(self.p3, "pack")
            else POINTFIX.synthetic(*self.p3).pack()
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, p1, p2, p3):
        self = cls.__new__(cls)
        self.p1 = p1
        self.p2 = p2
        self.p3 = p3
        return self


class TRAPEZOID(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.top, self.bottom = unpacker.unpack("=ii")
        self.left = LINEFIX(unpacker)
        unpacker.pad(LINEFIX)
        self.right = LINEFIX(unpacker)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=ii", self.top, self.bottom))
        buf.write(
            self.left.pack()
            if hasattr(self.left, "pack")
            else LINEFIX.synthetic(*self.left).pack()
        )
        buf.write(
            self.right.pack()
            if hasattr(self.right, "pack")
            else LINEFIX.synthetic(*self.right).pack()
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, top, bottom, left, right):
        self = cls.__new__(cls)
        self.top = top
        self.bottom = bottom
        self.left = left
        self.right = right
        return self


class GLYPHINFO(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.width, self.height, self.x, self.y, self.x_off, self.y_off = (
            unpacker.unpack("=HHhhhh")
        )
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=HHhhhh",
                self.width,
                self.height,
                self.x,
                self.y,
                self.x_off,
                self.y_off,
            )
        )
        return buf.getvalue()

    fixed_size = 12

    @classmethod
    def synthetic(cls, width, height, x, y, x_off, y_off):
        self = cls.__new__(cls)
        self.width = width
        self.height = height
        self.x = x
        self.y = y
        self.x_off = x_off
        self.y_off = y_off
        return self


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.major_version, self.minor_version = unpacker.unpack("=xx2x4xII16x")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class QueryPictFormatsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.num_formats,
            self.num_screens,
            self.num_depths,
            self.num_visuals,
            self.num_subpixel,
        ) = unpacker.unpack("=xx2x4xIIIII4x")
        self.formats = xcffib.List(unpacker, PICTFORMINFO, self.num_formats)
        unpacker.pad(PICTSCREEN)
        self.screens = xcffib.List(unpacker, PICTSCREEN, self.num_screens)
        unpacker.pad("I")
        self.subpixels = xcffib.List(unpacker, "I", self.num_subpixel)
        self.bufsize = unpacker.offset - base


class QueryPictFormatsCookie(xcffib.Cookie):
    reply_type = QueryPictFormatsReply


class QueryPictIndexValuesReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.num_values,) = unpacker.unpack("=xx2x4xI20x")
        self.values = xcffib.List(unpacker, INDEXVALUE, self.num_values)
        self.bufsize = unpacker.offset - base


class QueryPictIndexValuesCookie(xcffib.Cookie):
    reply_type = QueryPictIndexValuesReply


class TRANSFORM(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.matrix11,
            self.matrix12,
            self.matrix13,
            self.matrix21,
            self.matrix22,
            self.matrix23,
            self.matrix31,
            self.matrix32,
            self.matrix33,
        ) = unpacker.unpack("=iiiiiiiii")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=iiiiiiiii",
                self.matrix11,
                self.matrix12,
                self.matrix13,
                self.matrix21,
                self.matrix22,
                self.matrix23,
                self.matrix31,
                self.matrix32,
                self.matrix33,
            )
        )
        return buf.getvalue()

    fixed_size = 36

    @classmethod
    def synthetic(
        cls,
        matrix11,
        matrix12,
        matrix13,
        matrix21,
        matrix22,
        matrix23,
        matrix31,
        matrix32,
        matrix33,
    ):
        self = cls.__new__(cls)
        self.matrix11 = matrix11
        self.matrix12 = matrix12
        self.matrix13 = matrix13
        self.matrix21 = matrix21
        self.matrix22 = matrix22
        self.matrix23 = matrix23
        self.matrix31 = matrix31
        self.matrix32 = matrix32
        self.matrix33 = matrix33
        return self


class QueryFiltersReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.num_aliases, self.num_filters = unpacker.unpack("=xx2x4xII16x")
        self.aliases = xcffib.List(unpacker, "H", self.num_aliases)
        unpacker.pad(xproto.STR)
        self.filters = xcffib.List(unpacker, xproto.STR, self.num_filters)
        self.bufsize = unpacker.offset - base


class QueryFiltersCookie(xcffib.Cookie):
    reply_type = QueryFiltersReply


class ANIMCURSORELT(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.cursor, self.delay = unpacker.unpack("=II")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=II", self.cursor, self.delay))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, cursor, delay):
        self = cls.__new__(cls)
        self.cursor = cursor
        self.delay = delay
        return self


class SPANFIX(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.l, self.r, self.y = unpacker.unpack("=iii")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=iii", self.l, self.r, self.y))
        return buf.getvalue()

    fixed_size = 12

    @classmethod
    def synthetic(cls, l, r, y):
        self = cls.__new__(cls)
        self.l = l
        self.r = r
        self.y = y
        return self


class TRAP(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.top = SPANFIX(unpacker)
        unpacker.pad(SPANFIX)
        self.bot = SPANFIX(unpacker)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            self.top.pack()
            if hasattr(self.top, "pack")
            else SPANFIX.synthetic(*self.top).pack()
        )
        buf.write(
            self.bot.pack()
            if hasattr(self.bot, "pack")
            else SPANFIX.synthetic(*self.bot).pack()
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, top, bot):
        self = cls.__new__(cls)
        self.top = top
        self.bot = bot
        return self


class renderExtension(xcffib.Extension):
    def QueryVersion(self, client_major_version, client_minor_version, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", client_major_version, client_minor_version))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self, client_major_version, client_minor_version):
        return self.QueryVersion(
            client_major_version, client_minor_version, is_checked=True
        )

    def QueryVersionUnchecked(self, client_major_version, client_minor_version):
        return self.QueryVersion(
            client_major_version, client_minor_version, is_checked=False
        )

    def QueryPictFormats(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(1, buf, QueryPictFormatsCookie, is_checked=is_checked)

    def QueryPictFormatsChecked(self):
        return self.QueryPictFormats(is_checked=True)

    def QueryPictFormatsUnchecked(self):
        return self.QueryPictFormats(is_checked=False)

    def QueryPictIndexValues(self, format, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", format))
        return self.send_request(
            2, buf, QueryPictIndexValuesCookie, is_checked=is_checked
        )

    def QueryPictIndexValuesChecked(self, format):
        return self.QueryPictIndexValues(format, is_checked=True)

    def QueryPictIndexValuesUnchecked(self, format):
        return self.QueryPictIndexValues(format, is_checked=False)

    def CreatePicture(
        self, pid, drawable, format, value_mask, value_list, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIII", pid, drawable, format, value_mask))
        if value_mask & CP.Repeat:
            repeat = value_list.pop(0)
            buf.write(struct.pack("=I", repeat))
        if value_mask & CP.AlphaMap:
            alphamap = value_list.pop(0)
            buf.write(struct.pack("=I", alphamap))
        if value_mask & CP.AlphaXOrigin:
            alphaxorigin = value_list.pop(0)
            buf.write(struct.pack("=i", alphaxorigin))
        if value_mask & CP.AlphaYOrigin:
            alphayorigin = value_list.pop(0)
            buf.write(struct.pack("=i", alphayorigin))
        if value_mask & CP.ClipXOrigin:
            clipxorigin = value_list.pop(0)
            buf.write(struct.pack("=i", clipxorigin))
        if value_mask & CP.ClipYOrigin:
            clipyorigin = value_list.pop(0)
            buf.write(struct.pack("=i", clipyorigin))
        if value_mask & CP.ClipMask:
            clipmask = value_list.pop(0)
            buf.write(struct.pack("=I", clipmask))
        if value_mask & CP.GraphicsExposure:
            graphicsexposure = value_list.pop(0)
            buf.write(struct.pack("=I", graphicsexposure))
        if value_mask & CP.SubwindowMode:
            subwindowmode = value_list.pop(0)
            buf.write(struct.pack("=I", subwindowmode))
        if value_mask & CP.PolyEdge:
            polyedge = value_list.pop(0)
            buf.write(struct.pack("=I", polyedge))
        if value_mask & CP.PolyMode:
            polymode = value_list.pop(0)
            buf.write(struct.pack("=I", polymode))
        if value_mask & CP.Dither:
            dither = value_list.pop(0)
            buf.write(struct.pack("=I", dither))
        if value_mask & CP.ComponentAlpha:
            componentalpha = value_list.pop(0)
            buf.write(struct.pack("=I", componentalpha))
        return self.send_request(4, buf, is_checked=is_checked)

    def CreatePictureChecked(self, pid, drawable, format, value_mask, value_list):
        return self.CreatePicture(
            pid, drawable, format, value_mask, value_list, is_checked=True
        )

    def CreatePictureUnchecked(self, pid, drawable, format, value_mask, value_list):
        return self.CreatePicture(
            pid, drawable, format, value_mask, value_list, is_checked=False
        )

    def ChangePicture(self, picture, value_mask, value_list, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", picture, value_mask))
        if value_mask & CP.Repeat:
            repeat = value_list.pop(0)
            buf.write(struct.pack("=I", repeat))
        if value_mask & CP.AlphaMap:
            alphamap = value_list.pop(0)
            buf.write(struct.pack("=I", alphamap))
        if value_mask & CP.AlphaXOrigin:
            alphaxorigin = value_list.pop(0)
            buf.write(struct.pack("=i", alphaxorigin))
        if value_mask & CP.AlphaYOrigin:
            alphayorigin = value_list.pop(0)
            buf.write(struct.pack("=i", alphayorigin))
        if value_mask & CP.ClipXOrigin:
            clipxorigin = value_list.pop(0)
            buf.write(struct.pack("=i", clipxorigin))
        if value_mask & CP.ClipYOrigin:
            clipyorigin = value_list.pop(0)
            buf.write(struct.pack("=i", clipyorigin))
        if value_mask & CP.ClipMask:
            clipmask = value_list.pop(0)
            buf.write(struct.pack("=I", clipmask))
        if value_mask & CP.GraphicsExposure:
            graphicsexposure = value_list.pop(0)
            buf.write(struct.pack("=I", graphicsexposure))
        if value_mask & CP.SubwindowMode:
            subwindowmode = value_list.pop(0)
            buf.write(struct.pack("=I", subwindowmode))
        if value_mask & CP.PolyEdge:
            polyedge = value_list.pop(0)
            buf.write(struct.pack("=I", polyedge))
        if value_mask & CP.PolyMode:
            polymode = value_list.pop(0)
            buf.write(struct.pack("=I", polymode))
        if value_mask & CP.Dither:
            dither = value_list.pop(0)
            buf.write(struct.pack("=I", dither))
        if value_mask & CP.ComponentAlpha:
            componentalpha = value_list.pop(0)
            buf.write(struct.pack("=I", componentalpha))
        return self.send_request(5, buf, is_checked=is_checked)

    def ChangePictureChecked(self, picture, value_mask, value_list):
        return self.ChangePicture(picture, value_mask, value_list, is_checked=True)

    def ChangePictureUnchecked(self, picture, value_mask, value_list):
        return self.ChangePicture(picture, value_mask, value_list, is_checked=False)

    def SetPictureClipRectangles(
        self,
        picture,
        clip_x_origin,
        clip_y_origin,
        rectangles_len,
        rectangles,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIhh", picture, clip_x_origin, clip_y_origin))
        buf.write(xcffib.pack_list(rectangles, xproto.RECTANGLE))
        return self.send_request(6, buf, is_checked=is_checked)

    def SetPictureClipRectanglesChecked(
        self, picture, clip_x_origin, clip_y_origin, rectangles_len, rectangles
    ):
        return self.SetPictureClipRectangles(
            picture,
            clip_x_origin,
            clip_y_origin,
            rectangles_len,
            rectangles,
            is_checked=True,
        )

    def SetPictureClipRectanglesUnchecked(
        self, picture, clip_x_origin, clip_y_origin, rectangles_len, rectangles
    ):
        return self.SetPictureClipRectangles(
            picture,
            clip_x_origin,
            clip_y_origin,
            rectangles_len,
            rectangles,
            is_checked=False,
        )

    def FreePicture(self, picture, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", picture))
        return self.send_request(7, buf, is_checked=is_checked)

    def FreePictureChecked(self, picture):
        return self.FreePicture(picture, is_checked=True)

    def FreePictureUnchecked(self, picture):
        return self.FreePicture(picture, is_checked=False)

    def Composite(
        self,
        op,
        src,
        mask,
        dst,
        src_x,
        src_y,
        mask_x,
        mask_y,
        dst_x,
        dst_y,
        width,
        height,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xB3xIIIhhhhhhHH",
                op,
                src,
                mask,
                dst,
                src_x,
                src_y,
                mask_x,
                mask_y,
                dst_x,
                dst_y,
                width,
                height,
            )
        )
        return self.send_request(8, buf, is_checked=is_checked)

    def CompositeChecked(
        self,
        op,
        src,
        mask,
        dst,
        src_x,
        src_y,
        mask_x,
        mask_y,
        dst_x,
        dst_y,
        width,
        height,
    ):
        return self.Composite(
            op,
            src,
            mask,
            dst,
            src_x,
            src_y,
            mask_x,
            mask_y,
            dst_x,
            dst_y,
            width,
            height,
            is_checked=True,
        )

    def CompositeUnchecked(
        self,
        op,
        src,
        mask,
        dst,
        src_x,
        src_y,
        mask_x,
        mask_y,
        dst_x,
        dst_y,
        width,
        height,
    ):
        return self.Composite(
            op,
            src,
            mask,
            dst,
            src_x,
            src_y,
            mask_x,
            mask_y,
            dst_x,
            dst_y,
            width,
            height,
            is_checked=False,
        )

    def Trapezoids(
        self,
        op,
        src,
        dst,
        mask_format,
        src_x,
        src_y,
        traps_len,
        traps,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xB3xIIIhh", op, src, dst, mask_format, src_x, src_y))
        buf.write(xcffib.pack_list(traps, TRAPEZOID))
        return self.send_request(10, buf, is_checked=is_checked)

    def TrapezoidsChecked(
        self, op, src, dst, mask_format, src_x, src_y, traps_len, traps
    ):
        return self.Trapezoids(
            op, src, dst, mask_format, src_x, src_y, traps_len, traps, is_checked=True
        )

    def TrapezoidsUnchecked(
        self, op, src, dst, mask_format, src_x, src_y, traps_len, traps
    ):
        return self.Trapezoids(
            op, src, dst, mask_format, src_x, src_y, traps_len, traps, is_checked=False
        )

    def Triangles(
        self,
        op,
        src,
        dst,
        mask_format,
        src_x,
        src_y,
        triangles_len,
        triangles,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xB3xIIIhh", op, src, dst, mask_format, src_x, src_y))
        buf.write(xcffib.pack_list(triangles, TRIANGLE))
        return self.send_request(11, buf, is_checked=is_checked)

    def TrianglesChecked(
        self, op, src, dst, mask_format, src_x, src_y, triangles_len, triangles
    ):
        return self.Triangles(
            op,
            src,
            dst,
            mask_format,
            src_x,
            src_y,
            triangles_len,
            triangles,
            is_checked=True,
        )

    def TrianglesUnchecked(
        self, op, src, dst, mask_format, src_x, src_y, triangles_len, triangles
    ):
        return self.Triangles(
            op,
            src,
            dst,
            mask_format,
            src_x,
            src_y,
            triangles_len,
            triangles,
            is_checked=False,
        )

    def TriStrip(
        self,
        op,
        src,
        dst,
        mask_format,
        src_x,
        src_y,
        points_len,
        points,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xB3xIIIhh", op, src, dst, mask_format, src_x, src_y))
        buf.write(xcffib.pack_list(points, POINTFIX))
        return self.send_request(12, buf, is_checked=is_checked)

    def TriStripChecked(
        self, op, src, dst, mask_format, src_x, src_y, points_len, points
    ):
        return self.TriStrip(
            op, src, dst, mask_format, src_x, src_y, points_len, points, is_checked=True
        )

    def TriStripUnchecked(
        self, op, src, dst, mask_format, src_x, src_y, points_len, points
    ):
        return self.TriStrip(
            op,
            src,
            dst,
            mask_format,
            src_x,
            src_y,
            points_len,
            points,
            is_checked=False,
        )

    def TriFan(
        self,
        op,
        src,
        dst,
        mask_format,
        src_x,
        src_y,
        points_len,
        points,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xB3xIIIhh", op, src, dst, mask_format, src_x, src_y))
        buf.write(xcffib.pack_list(points, POINTFIX))
        return self.send_request(13, buf, is_checked=is_checked)

    def TriFanChecked(
        self, op, src, dst, mask_format, src_x, src_y, points_len, points
    ):
        return self.TriFan(
            op, src, dst, mask_format, src_x, src_y, points_len, points, is_checked=True
        )

    def TriFanUnchecked(
        self, op, src, dst, mask_format, src_x, src_y, points_len, points
    ):
        return self.TriFan(
            op,
            src,
            dst,
            mask_format,
            src_x,
            src_y,
            points_len,
            points,
            is_checked=False,
        )

    def CreateGlyphSet(self, gsid, format, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", gsid, format))
        return self.send_request(17, buf, is_checked=is_checked)

    def CreateGlyphSetChecked(self, gsid, format):
        return self.CreateGlyphSet(gsid, format, is_checked=True)

    def CreateGlyphSetUnchecked(self, gsid, format):
        return self.CreateGlyphSet(gsid, format, is_checked=False)

    def ReferenceGlyphSet(self, gsid, existing, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", gsid, existing))
        return self.send_request(18, buf, is_checked=is_checked)

    def ReferenceGlyphSetChecked(self, gsid, existing):
        return self.ReferenceGlyphSet(gsid, existing, is_checked=True)

    def ReferenceGlyphSetUnchecked(self, gsid, existing):
        return self.ReferenceGlyphSet(gsid, existing, is_checked=False)

    def FreeGlyphSet(self, glyphset, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", glyphset))
        return self.send_request(19, buf, is_checked=is_checked)

    def FreeGlyphSetChecked(self, glyphset):
        return self.FreeGlyphSet(glyphset, is_checked=True)

    def FreeGlyphSetUnchecked(self, glyphset):
        return self.FreeGlyphSet(glyphset, is_checked=False)

    def AddGlyphs(
        self, glyphset, glyphs_len, glyphids, glyphs, data_len, data, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", glyphset, glyphs_len))
        buf.write(xcffib.pack_list(glyphids, "I"))
        buf.write(xcffib.pack_list(glyphs, GLYPHINFO))
        buf.write(xcffib.pack_list(data, "B"))
        return self.send_request(20, buf, is_checked=is_checked)

    def AddGlyphsChecked(self, glyphset, glyphs_len, glyphids, glyphs, data_len, data):
        return self.AddGlyphs(
            glyphset, glyphs_len, glyphids, glyphs, data_len, data, is_checked=True
        )

    def AddGlyphsUnchecked(
        self, glyphset, glyphs_len, glyphids, glyphs, data_len, data
    ):
        return self.AddGlyphs(
            glyphset, glyphs_len, glyphids, glyphs, data_len, data, is_checked=False
        )

    def FreeGlyphs(self, glyphset, glyphs_len, glyphs, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", glyphset))
        buf.write(xcffib.pack_list(glyphs, "I"))
        return self.send_request(22, buf, is_checked=is_checked)

    def FreeGlyphsChecked(self, glyphset, glyphs_len, glyphs):
        return self.FreeGlyphs(glyphset, glyphs_len, glyphs, is_checked=True)

    def FreeGlyphsUnchecked(self, glyphset, glyphs_len, glyphs):
        return self.FreeGlyphs(glyphset, glyphs_len, glyphs, is_checked=False)

    def CompositeGlyphs8(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xB3xIIIIhh", op, src, dst, mask_format, glyphset, src_x, src_y
            )
        )
        buf.write(xcffib.pack_list(glyphcmds, "B"))
        return self.send_request(23, buf, is_checked=is_checked)

    def CompositeGlyphs8Checked(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
    ):
        return self.CompositeGlyphs8(
            op,
            src,
            dst,
            mask_format,
            glyphset,
            src_x,
            src_y,
            glyphcmds_len,
            glyphcmds,
            is_checked=True,
        )

    def CompositeGlyphs8Unchecked(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
    ):
        return self.CompositeGlyphs8(
            op,
            src,
            dst,
            mask_format,
            glyphset,
            src_x,
            src_y,
            glyphcmds_len,
            glyphcmds,
            is_checked=False,
        )

    def CompositeGlyphs16(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xB3xIIIIhh", op, src, dst, mask_format, glyphset, src_x, src_y
            )
        )
        buf.write(xcffib.pack_list(glyphcmds, "B"))
        return self.send_request(24, buf, is_checked=is_checked)

    def CompositeGlyphs16Checked(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
    ):
        return self.CompositeGlyphs16(
            op,
            src,
            dst,
            mask_format,
            glyphset,
            src_x,
            src_y,
            glyphcmds_len,
            glyphcmds,
            is_checked=True,
        )

    def CompositeGlyphs16Unchecked(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
    ):
        return self.CompositeGlyphs16(
            op,
            src,
            dst,
            mask_format,
            glyphset,
            src_x,
            src_y,
            glyphcmds_len,
            glyphcmds,
            is_checked=False,
        )

    def CompositeGlyphs32(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xB3xIIIIhh", op, src, dst, mask_format, glyphset, src_x, src_y
            )
        )
        buf.write(xcffib.pack_list(glyphcmds, "B"))
        return self.send_request(25, buf, is_checked=is_checked)

    def CompositeGlyphs32Checked(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
    ):
        return self.CompositeGlyphs32(
            op,
            src,
            dst,
            mask_format,
            glyphset,
            src_x,
            src_y,
            glyphcmds_len,
            glyphcmds,
            is_checked=True,
        )

    def CompositeGlyphs32Unchecked(
        self,
        op,
        src,
        dst,
        mask_format,
        glyphset,
        src_x,
        src_y,
        glyphcmds_len,
        glyphcmds,
    ):
        return self.CompositeGlyphs32(
            op,
            src,
            dst,
            mask_format,
            glyphset,
            src_x,
            src_y,
            glyphcmds_len,
            glyphcmds,
            is_checked=False,
        )

    def FillRectangles(self, op, dst, color, rects_len, rects, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xB3xI", op, dst))
        buf.write(
            color.pack() if hasattr(color, "pack") else COLOR.synthetic(*color).pack()
        )
        buf.write(xcffib.pack_list(rects, xproto.RECTANGLE))
        return self.send_request(26, buf, is_checked=is_checked)

    def FillRectanglesChecked(self, op, dst, color, rects_len, rects):
        return self.FillRectangles(op, dst, color, rects_len, rects, is_checked=True)

    def FillRectanglesUnchecked(self, op, dst, color, rects_len, rects):
        return self.FillRectangles(op, dst, color, rects_len, rects, is_checked=False)

    def CreateCursor(self, cid, source, x, y, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIHH", cid, source, x, y))
        return self.send_request(27, buf, is_checked=is_checked)

    def CreateCursorChecked(self, cid, source, x, y):
        return self.CreateCursor(cid, source, x, y, is_checked=True)

    def CreateCursorUnchecked(self, cid, source, x, y):
        return self.CreateCursor(cid, source, x, y, is_checked=False)

    def SetPictureTransform(self, picture, transform, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", picture))
        buf.write(
            transform.pack()
            if hasattr(transform, "pack")
            else TRANSFORM.synthetic(*transform).pack()
        )
        return self.send_request(28, buf, is_checked=is_checked)

    def SetPictureTransformChecked(self, picture, transform):
        return self.SetPictureTransform(picture, transform, is_checked=True)

    def SetPictureTransformUnchecked(self, picture, transform):
        return self.SetPictureTransform(picture, transform, is_checked=False)

    def QueryFilters(self, drawable, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", drawable))
        return self.send_request(29, buf, QueryFiltersCookie, is_checked=is_checked)

    def QueryFiltersChecked(self, drawable):
        return self.QueryFilters(drawable, is_checked=True)

    def QueryFiltersUnchecked(self, drawable):
        return self.QueryFilters(drawable, is_checked=False)

    def SetPictureFilter(
        self, picture, filter_len, filter, values_len, values, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIH2x", picture, filter_len))
        buf.write(xcffib.pack_list(filter, "c"))
        buf.write(
            struct.pack(
                "=4x",
            )
        )
        buf.write(xcffib.pack_list(values, "i"))
        return self.send_request(30, buf, is_checked=is_checked)

    def SetPictureFilterChecked(self, picture, filter_len, filter, values_len, values):
        return self.SetPictureFilter(
            picture, filter_len, filter, values_len, values, is_checked=True
        )

    def SetPictureFilterUnchecked(
        self, picture, filter_len, filter, values_len, values
    ):
        return self.SetPictureFilter(
            picture, filter_len, filter, values_len, values, is_checked=False
        )

    def CreateAnimCursor(self, cid, cursors_len, cursors, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", cid))
        buf.write(xcffib.pack_list(cursors, ANIMCURSORELT))
        return self.send_request(31, buf, is_checked=is_checked)

    def CreateAnimCursorChecked(self, cid, cursors_len, cursors):
        return self.CreateAnimCursor(cid, cursors_len, cursors, is_checked=True)

    def CreateAnimCursorUnchecked(self, cid, cursors_len, cursors):
        return self.CreateAnimCursor(cid, cursors_len, cursors, is_checked=False)

    def AddTraps(self, picture, x_off, y_off, traps_len, traps, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIhh", picture, x_off, y_off))
        buf.write(xcffib.pack_list(traps, TRAP))
        return self.send_request(32, buf, is_checked=is_checked)

    def AddTrapsChecked(self, picture, x_off, y_off, traps_len, traps):
        return self.AddTraps(picture, x_off, y_off, traps_len, traps, is_checked=True)

    def AddTrapsUnchecked(self, picture, x_off, y_off, traps_len, traps):
        return self.AddTraps(picture, x_off, y_off, traps_len, traps, is_checked=False)

    def CreateSolidFill(self, picture, color, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", picture))
        buf.write(
            color.pack() if hasattr(color, "pack") else COLOR.synthetic(*color).pack()
        )
        return self.send_request(33, buf, is_checked=is_checked)

    def CreateSolidFillChecked(self, picture, color):
        return self.CreateSolidFill(picture, color, is_checked=True)

    def CreateSolidFillUnchecked(self, picture, color):
        return self.CreateSolidFill(picture, color, is_checked=False)

    def CreateLinearGradient(
        self, picture, p1, p2, num_stops, stops, colors, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", picture))
        buf.write(p1.pack() if hasattr(p1, "pack") else POINTFIX.synthetic(*p1).pack())
        buf.write(p2.pack() if hasattr(p2, "pack") else POINTFIX.synthetic(*p2).pack())
        buf.write(struct.pack("=I", num_stops))
        buf.write(xcffib.pack_list(stops, "i"))
        buf.write(xcffib.pack_list(colors, COLOR))
        return self.send_request(34, buf, is_checked=is_checked)

    def CreateLinearGradientChecked(self, picture, p1, p2, num_stops, stops, colors):
        return self.CreateLinearGradient(
            picture, p1, p2, num_stops, stops, colors, is_checked=True
        )

    def CreateLinearGradientUnchecked(self, picture, p1, p2, num_stops, stops, colors):
        return self.CreateLinearGradient(
            picture, p1, p2, num_stops, stops, colors, is_checked=False
        )

    def CreateRadialGradient(
        self,
        picture,
        inner,
        outer,
        inner_radius,
        outer_radius,
        num_stops,
        stops,
        colors,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", picture))
        buf.write(
            inner.pack()
            if hasattr(inner, "pack")
            else POINTFIX.synthetic(*inner).pack()
        )
        buf.write(
            outer.pack()
            if hasattr(outer, "pack")
            else POINTFIX.synthetic(*outer).pack()
        )
        buf.write(struct.pack("=i", inner_radius))
        buf.write(struct.pack("=i", outer_radius))
        buf.write(struct.pack("=I", num_stops))
        buf.write(xcffib.pack_list(stops, "i"))
        buf.write(xcffib.pack_list(colors, COLOR))
        return self.send_request(35, buf, is_checked=is_checked)

    def CreateRadialGradientChecked(
        self,
        picture,
        inner,
        outer,
        inner_radius,
        outer_radius,
        num_stops,
        stops,
        colors,
    ):
        return self.CreateRadialGradient(
            picture,
            inner,
            outer,
            inner_radius,
            outer_radius,
            num_stops,
            stops,
            colors,
            is_checked=True,
        )

    def CreateRadialGradientUnchecked(
        self,
        picture,
        inner,
        outer,
        inner_radius,
        outer_radius,
        num_stops,
        stops,
        colors,
    ):
        return self.CreateRadialGradient(
            picture,
            inner,
            outer,
            inner_radius,
            outer_radius,
            num_stops,
            stops,
            colors,
            is_checked=False,
        )

    def CreateConicalGradient(
        self, picture, center, angle, num_stops, stops, colors, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", picture))
        buf.write(
            center.pack()
            if hasattr(center, "pack")
            else POINTFIX.synthetic(*center).pack()
        )
        buf.write(struct.pack("=i", angle))
        buf.write(struct.pack("=I", num_stops))
        buf.write(xcffib.pack_list(stops, "i"))
        buf.write(xcffib.pack_list(colors, COLOR))
        return self.send_request(36, buf, is_checked=is_checked)

    def CreateConicalGradientChecked(
        self, picture, center, angle, num_stops, stops, colors
    ):
        return self.CreateConicalGradient(
            picture, center, angle, num_stops, stops, colors, is_checked=True
        )

    def CreateConicalGradientUnchecked(
        self, picture, center, angle, num_stops, stops, colors
    ):
        return self.CreateConicalGradient(
            picture, center, angle, num_stops, stops, colors, is_checked=False
        )


xcffib._add_ext(key, renderExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 2
MINOR_VERSION = 2
key = xcffib.ExtensionKey("XFree86-VidModeExtension")
_events = {}
_errors = {}


class ModeFlag:
    Positive_HSync = 1 << 0
    Negative_HSync = 1 << 1
    Positive_VSync = 1 << 2
    Negative_VSync = 1 << 3
    Interlace = 1 << 4
    Composite_Sync = 1 << 5
    Positive_CSync = 1 << 6
    Negative_CSync = 1 << 7
    HSkew = 1 << 8
    Broadcast = 1 << 9
    Pixmux = 1 << 10
    Double_Clock = 1 << 11
    Half_Clock = 1 << 12


class ClockFlag:
    Programable = 1 << 0


class Permission:
    Read = 1 << 0
    Write = 1 << 1


class ModeInfo(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.dotclock,
            self.hdisplay,
            self.hsyncstart,
            self.hsyncend,
            self.htotal,
            self.hskew,
            self.vdisplay,
            self.vsyncstart,
            self.vsyncend,
            self.vtotal,
            self.flags,
            self.privsize,
        ) = unpacker.unpack("=IHHHHIHHHH4xI12xI")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=IHHHHIHHHH4xI12xI",
                self.dotclock,
                self.hdisplay,
                self.hsyncstart,
                self.hsyncend,
                self.htotal,
                self.hskew,
                self.vdisplay,
                self.vsyncstart,
                self.vsyncend,
                self.vtotal,
                self.flags,
                self.privsize,
            )
        )
        return buf.getvalue()

    fixed_size = 48

    @classmethod
    def synthetic(
        cls,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
    ):
        self = cls.__new__(cls)
        self.dotclock = dotclock
        self.hdisplay = hdisplay
        self.hsyncstart = hsyncstart
        self.hsyncend = hsyncend
        self.htotal = htotal
        self.hskew = hskew
        self.vdisplay = vdisplay
        self.vsyncstart = vsyncstart
        self.vsyncend = vsyncend
        self.vtotal = vtotal
        self.flags = flags
        self.privsize = privsize
        return self


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.major_version, self.minor_version = unpacker.unpack("=xx2x4xHH")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class GetModeLineReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.dotclock,
            self.hdisplay,
            self.hsyncstart,
            self.hsyncend,
            self.htotal,
            self.hskew,
            self.vdisplay,
            self.vsyncstart,
            self.vsyncend,
            self.vtotal,
            self.flags,
            self.privsize,
        ) = unpacker.unpack("=xx2x4xIHHHHHHHHH2xI12xI")
        self.private = xcffib.List(unpacker, "B", self.privsize)
        self.bufsize = unpacker.offset - base


class GetModeLineCookie(xcffib.Cookie):
    reply_type = GetModeLineReply


class GetMonitorReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.vendor_length, self.model_length, self.num_hsync, self.num_vsync = (
            unpacker.unpack("=xx2x4xBBBB20x")
        )
        self.hsync = xcffib.List(unpacker, "I", self.num_hsync)
        unpacker.pad("I")
        self.vsync = xcffib.List(unpacker, "I", self.num_vsync)
        unpacker.pad("c")
        self.vendor = xcffib.List(unpacker, "c", self.vendor_length)
        unpacker.pad("c")
        self.alignment_pad = xcffib.List(
            unpacker, "c", ((self.vendor_length + 3) & (~3)) - self.vendor_length
        )
        unpacker.pad("c")
        self.model = xcffib.List(unpacker, "c", self.model_length)
        self.bufsize = unpacker.offset - base


class GetMonitorCookie(xcffib.Cookie):
    reply_type = GetMonitorReply


class GetAllModeLinesReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.modecount,) = unpacker.unpack("=xx2x4xI20x")
        self.modeinfo = xcffib.List(unpacker, ModeInfo, self.modecount)
        self.bufsize = unpacker.offset - base


class GetAllModeLinesCookie(xcffib.Cookie):
    reply_type = GetAllModeLinesReply


class ValidateModeLineReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.status,) = unpacker.unpack("=xx2x4xI20x")
        self.bufsize = unpacker.offset - base


class ValidateModeLineCookie(xcffib.Cookie):
    reply_type = ValidateModeLineReply


class GetViewPortReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.x, self.y = unpacker.unpack("=xx2x4xII16x")
        self.bufsize = unpacker.offset - base


class GetViewPortCookie(xcffib.Cookie):
    reply_type = GetViewPortReply


class GetDotClocksReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.flags, self.clocks, self.maxclocks = unpacker.unpack("=xx2x4xIII12x")
        self.clock = xcffib.List(unpacker, "I", (1 - (self.flags & 1)) * self.clocks)
        self.bufsize = unpacker.offset - base


class GetDotClocksCookie(xcffib.Cookie):
    reply_type = GetDotClocksReply


class GetGammaReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.red, self.green, self.blue = unpacker.unpack("=xx2x4xIII12x")
        self.bufsize = unpacker.offset - base


class GetGammaCookie(xcffib.Cookie):
    reply_type = GetGammaReply


class GetGammaRampReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.size,) = unpacker.unpack("=xx2x4xH22x")
        self.red = xcffib.List(unpacker, "H", (self.size + 1) & (~1))
        unpacker.pad("H")
        self.green = xcffib.List(unpacker, "H", (self.size + 1) & (~1))
        unpacker.pad("H")
        self.blue = xcffib.List(unpacker, "H", (self.size + 1) & (~1))
        self.bufsize = unpacker.offset - base


class GetGammaRampCookie(xcffib.Cookie):
    reply_type = GetGammaRampReply


class GetGammaRampSizeReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.size,) = unpacker.unpack("=xx2x4xH22x")
        self.bufsize = unpacker.offset - base


class GetGammaRampSizeCookie(xcffib.Cookie):
    reply_type = GetGammaRampSizeReply


class GetPermissionsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.permissions,) = unpacker.unpack("=xx2x4xI20x")
        self.bufsize = unpacker.offset - base


class GetPermissionsCookie(xcffib.Cookie):
    reply_type = GetPermissionsReply


class BadClockError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadBadClock = BadClockError
_errors[0] = BadClockError


class BadHTimingsError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 1))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadBadHTimings = BadHTimingsError
_errors[1] = BadHTimingsError


class BadVTimingsError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 2))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadBadVTimings = BadVTimingsError
_errors[2] = BadVTimingsError


class ModeUnsuitableError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 3))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadModeUnsuitable = ModeUnsuitableError
_errors[3] = ModeUnsuitableError


class ExtensionDisabledError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 4))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadExtensionDisabled = ExtensionDisabledError
_errors[4] = ExtensionDisabledError


class ClientNotLocalError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 5))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadClientNotLocal = ClientNotLocalError
_errors[5] = ClientNotLocalError


class ZoomLockedError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 6))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadZoomLocked = ZoomLockedError
_errors[6] = ZoomLockedError


class xf86vidmodeExtension(xcffib.Extension):
    def QueryVersion(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self):
        return self.QueryVersion(is_checked=True)

    def QueryVersionUnchecked(self):
        return self.QueryVersion(is_checked=False)

    def GetModeLine(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(1, buf, GetModeLineCookie, is_checked=is_checked)

    def GetModeLineChecked(self, screen):
        return self.GetModeLine(screen, is_checked=True)

    def GetModeLineUnchecked(self, screen):
        return self.GetModeLine(screen, is_checked=False)

    def ModModeLine(
        self,
        screen,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIHHHHHHHHH2xI12xI",
                screen,
                hdisplay,
                hsyncstart,
                hsyncend,
                htotal,
                hskew,
                vdisplay,
                vsyncstart,
                vsyncend,
                vtotal,
                flags,
                privsize,
            )
        )
        buf.write(xcffib.pack_list(private, "B"))
        return self.send_request(2, buf, is_checked=is_checked)

    def ModModeLineChecked(
        self,
        screen,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.ModModeLine(
            screen,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=True,
        )

    def ModModeLineUnchecked(
        self,
        screen,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.ModModeLine(
            screen,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=False,
        )

    def SwitchMode(self, screen, zoom, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", screen, zoom))
        return self.send_request(3, buf, is_checked=is_checked)

    def SwitchModeChecked(self, screen, zoom):
        return self.SwitchMode(screen, zoom, is_checked=True)

    def SwitchModeUnchecked(self, screen, zoom):
        return self.SwitchMode(screen, zoom, is_checked=False)

    def GetMonitor(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(4, buf, GetMonitorCookie, is_checked=is_checked)

    def GetMonitorChecked(self, screen):
        return self.GetMonitor(screen, is_checked=True)

    def GetMonitorUnchecked(self, screen):
        return self.GetMonitor(screen, is_checked=False)

    def LockModeSwitch(self, screen, lock, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", screen, lock))
        return self.send_request(5, buf, is_checked=is_checked)

    def LockModeSwitchChecked(self, screen, lock):
        return self.LockModeSwitch(screen, lock, is_checked=True)

    def LockModeSwitchUnchecked(self, screen, lock):
        return self.LockModeSwitch(screen, lock, is_checked=False)

    def GetAllModeLines(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(6, buf, GetAllModeLinesCookie, is_checked=is_checked)

    def GetAllModeLinesChecked(self, screen):
        return self.GetAllModeLines(screen, is_checked=True)

    def GetAllModeLinesUnchecked(self, screen):
        return self.GetAllModeLines(screen, is_checked=False)

    def AddModeLine(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        after_dotclock,
        after_hdisplay,
        after_hsyncstart,
        after_hsyncend,
        after_htotal,
        after_hskew,
        after_vdisplay,
        after_vsyncstart,
        after_vsyncend,
        after_vtotal,
        after_flags,
        private,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIHHHHHHHHH2xI12xIIHHHHHHHHH2xI12x",
                screen,
                dotclock,
                hdisplay,
                hsyncstart,
                hsyncend,
                htotal,
                hskew,
                vdisplay,
                vsyncstart,
                vsyncend,
                vtotal,
                flags,
                privsize,
                after_dotclock,
                after_hdisplay,
                after_hsyncstart,
                after_hsyncend,
                after_htotal,
                after_hskew,
                after_vdisplay,
                after_vsyncstart,
                after_vsyncend,
                after_vtotal,
                after_flags,
            )
        )
        buf.write(xcffib.pack_list(private, "B"))
        return self.send_request(7, buf, is_checked=is_checked)

    def AddModeLineChecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        after_dotclock,
        after_hdisplay,
        after_hsyncstart,
        after_hsyncend,
        after_htotal,
        after_hskew,
        after_vdisplay,
        after_vsyncstart,
        after_vsyncend,
        after_vtotal,
        after_flags,
        private,
    ):
        return self.AddModeLine(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            after_dotclock,
            after_hdisplay,
            after_hsyncstart,
            after_hsyncend,
            after_htotal,
            after_hskew,
            after_vdisplay,
            after_vsyncstart,
            after_vsyncend,
            after_vtotal,
            after_flags,
            private,
            is_checked=True,
        )

    def AddModeLineUnchecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        after_dotclock,
        after_hdisplay,
        after_hsyncstart,
        after_hsyncend,
        after_htotal,
        after_hskew,
        after_vdisplay,
        after_vsyncstart,
        after_vsyncend,
        after_vtotal,
        after_flags,
        private,
    ):
        return self.AddModeLine(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            after_dotclock,
            after_hdisplay,
            after_hsyncstart,
            after_hsyncend,
            after_htotal,
            after_hskew,
            after_vdisplay,
            after_vsyncstart,
            after_vsyncend,
            after_vtotal,
            after_flags,
            private,
            is_checked=False,
        )

    def DeleteModeLine(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIHHHHHHHHH2xI12xI",
                screen,
                dotclock,
                hdisplay,
                hsyncstart,
                hsyncend,
                htotal,
                hskew,
                vdisplay,
                vsyncstart,
                vsyncend,
                vtotal,
                flags,
                privsize,
            )
        )
        buf.write(xcffib.pack_list(private, "B"))
        return self.send_request(8, buf, is_checked=is_checked)

    def DeleteModeLineChecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.DeleteModeLine(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=True,
        )

    def DeleteModeLineUnchecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.DeleteModeLine(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=False,
        )

    def ValidateModeLine(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
        is_checked=True,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIHHHHHHHHH2xI12xI",
                screen,
                dotclock,
                hdisplay,
                hsyncstart,
                hsyncend,
                htotal,
                hskew,
                vdisplay,
                vsyncstart,
                vsyncend,
                vtotal,
                flags,
                privsize,
            )
        )
        buf.write(xcffib.pack_list(private, "B"))
        return self.send_request(9, buf, ValidateModeLineCookie, is_checked=is_checked)

    def ValidateModeLineChecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.ValidateModeLine(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=True,
        )

    def ValidateModeLineUnchecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.ValidateModeLine(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=False,
        )

    def SwitchToMode(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIHHHHHHHHH2xI12xI",
                screen,
                dotclock,
                hdisplay,
                hsyncstart,
                hsyncend,
                htotal,
                hskew,
                vdisplay,
                vsyncstart,
                vsyncend,
                vtotal,
                flags,
                privsize,
            )
        )
        buf.write(xcffib.pack_list(private, "B"))
        return self.send_request(10, buf, is_checked=is_checked)

    def SwitchToModeChecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.SwitchToMode(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=True,
        )

    def SwitchToModeUnchecked(
        self,
        screen,
        dotclock,
        hdisplay,
        hsyncstart,
        hsyncend,
        htotal,
        hskew,
        vdisplay,
        vsyncstart,
        vsyncend,
        vtotal,
        flags,
        privsize,
        private,
    ):
        return self.SwitchToMode(
            screen,
            dotclock,
            hdisplay,
            hsyncstart,
            hsyncend,
            htotal,
            hskew,
            vdisplay,
            vsyncstart,
            vsyncend,
            vtotal,
            flags,
            privsize,
            private,
            is_checked=False,
        )

    def GetViewPort(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(11, buf, GetViewPortCookie, is_checked=is_checked)

    def GetViewPortChecked(self, screen):
        return self.GetViewPort(screen, is_checked=True)

    def GetViewPortUnchecked(self, screen):
        return self.GetViewPort(screen, is_checked=False)

    def SetViewPort(self, screen, x, y, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2xII", screen, x, y))
        return self.send_request(12, buf, is_checked=is_checked)

    def SetViewPortChecked(self, screen, x, y):
        return self.SetViewPort(screen, x, y, is_checked=True)

    def SetViewPortUnchecked(self, screen, x, y):
        return self.SetViewPort(screen, x, y, is_checked=False)

    def GetDotClocks(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(13, buf, GetDotClocksCookie, is_checked=is_checked)

    def GetDotClocksChecked(self, screen):
        return self.GetDotClocks(screen, is_checked=True)

    def GetDotClocksUnchecked(self, screen):
        return self.GetDotClocks(screen, is_checked=False)

    def SetClientVersion(self, major, minor, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", major, minor))
        return self.send_request(14, buf, is_checked=is_checked)

    def SetClientVersionChecked(self, major, minor):
        return self.SetClientVersion(major, minor, is_checked=True)

    def SetClientVersionUnchecked(self, major, minor):
        return self.SetClientVersion(major, minor, is_checked=False)

    def SetGamma(self, screen, red, green, blue, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2xIII12x", screen, red, green, blue))
        return self.send_request(15, buf, is_checked=is_checked)

    def SetGammaChecked(self, screen, red, green, blue):
        return self.SetGamma(screen, red, green, blue, is_checked=True)

    def SetGammaUnchecked(self, screen, red, green, blue):
        return self.SetGamma(screen, red, green, blue, is_checked=False)

    def GetGamma(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH26x", screen))
        return self.send_request(16, buf, GetGammaCookie, is_checked=is_checked)

    def GetGammaChecked(self, screen):
        return self.GetGamma(screen, is_checked=True)

    def GetGammaUnchecked(self, screen):
        return self.GetGamma(screen, is_checked=False)

    def GetGammaRamp(self, screen, size, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", screen, size))
        return self.send_request(17, buf, GetGammaRampCookie, is_checked=is_checked)

    def GetGammaRampChecked(self, screen, size):
        return self.GetGammaRamp(screen, size, is_checked=True)

    def GetGammaRampUnchecked(self, screen, size):
        return self.GetGammaRamp(screen, size, is_checked=False)

    def SetGammaRamp(self, screen, size, red, green, blue, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", screen, size))
        buf.write(xcffib.pack_list(red, "H"))
        buf.write(xcffib.pack_list(green, "H"))
        buf.write(xcffib.pack_list(blue, "H"))
        return self.send_request(18, buf, is_checked=is_checked)

    def SetGammaRampChecked(self, screen, size, red, green, blue):
        return self.SetGammaRamp(screen, size, red, green, blue, is_checked=True)

    def SetGammaRampUnchecked(self, screen, size, red, green, blue):
        return self.SetGammaRamp(screen, size, red, green, blue, is_checked=False)

    def GetGammaRampSize(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(19, buf, GetGammaRampSizeCookie, is_checked=is_checked)

    def GetGammaRampSizeChecked(self, screen):
        return self.GetGammaRampSize(screen, is_checked=True)

    def GetGammaRampSizeUnchecked(self, screen):
        return self.GetGammaRampSize(screen, is_checked=False)

    def GetPermissions(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xH2x", screen))
        return self.send_request(20, buf, GetPermissionsCookie, is_checked=is_checked)

    def GetPermissionsChecked(self, screen):
        return self.GetPermissions(screen, is_checked=True)

    def GetPermissionsUnchecked(self, screen):
        return self.GetPermissions(screen, is_checked=False)


xcffib._add_ext(key, xf86vidmodeExtension, _events, _errors)



import xcffib
import xcffib.xproto
import xcffib.xtest


def test_fakeinput(xcffib_test):
    xtest = xcffib_test.conn(xcffib.xtest.key)

    setup = xcffib_test.conn.get_setup()
    screen = setup.roots[0]

    def test(x, y):
        # motion
        xtest.FakeInput(6, 0, xcffib.xproto.Time.CurrentTime, screen.root, x, y, 0)

        # press
        xtest.FakeInput(4, 1, xcffib.xproto.Time.CurrentTime, screen.root, 0, 0, 0)

        # release
        xtest.FakeInput(5, 1, xcffib.xproto.Time.CurrentTime, screen.root, 2, 2, 0)
        xcffib_test.conn.flush()

    test(50, 10)

    # we shouldn't get any errors
    xcffib_test.conn.poll_for_event()



import xcffib
import xcffib.render
import xcffib.xproto
from xcffib.xproto import CW, EventMask, ExposeEvent, WindowClass

WIDTH = 50
HEIGHT = 50


def double_to_fixed(num):
    return int(num * 65536)


def find_format(screen, depth, visual):
    for d in screen.depths:
        if d.depth == depth:
            for v in d.visuals:
                if v.visual == visual:
                    return v.format


class TestConnection:
    def test_CreateLinearGradient(self, xproto_test):
        conn = xproto_test.conn
        setup = conn.get_setup()
        root = setup.roots[0].root
        depth = setup.roots[0].root_depth
        visual = setup.roots[0].root_visual
        black = setup.roots[0].black_pixel
        conn.render = conn(xcffib.render.key)
        conn.render.QueryVersion(
            xcffib.render.MAJOR_VERSION, xcffib.render.MINOR_VERSION
        )

        window = conn.generate_id()
        conn.core.CreateWindow(
            depth,
            window,
            root,
            0,
            0,
            640,
            480,
            0,
            WindowClass.InputOutput,
            visual,
            CW.BackPixel | CW.EventMask,
            [black, EventMask.Exposure | EventMask.KeyPress],
        )

        conn.core.MapWindow(window)
        conn.flush()

        while True:
            event = conn.wait_for_event()
            if isinstance(event, ExposeEvent):
                cookie = conn.render.QueryPictFormats()
                reply = cookie.reply()
                fmt = find_format(reply.screens[0], depth, visual)

                # Create picture from the window
                pic_window = conn.generate_id()
                conn.render.CreatePicture(pic_window, window, fmt, 0, [])

                # Create the linear gradient
                pic_gradient = conn.generate_id()
                conn.render.CreateLinearGradientChecked(
                    pic_gradient,
                    xcffib.render.POINTFIX.synthetic(0, 0),
                    xcffib.render.POINTFIX.synthetic(
                        double_to_fixed(WIDTH), double_to_fixed(HEIGHT)
                    ),
                    2,
                    [0, double_to_fixed(1)],
                    [
                        xcffib.render.COLOR.synthetic(0, 0, 0, 0xFFFF),  # Solid black
                        xcffib.render.COLOR.synthetic(
                            0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF
                        ),  # Solid white
                    ],
                ).check()

                # Render the gradient onto the window
                conn.render.Composite(
                    xcffib.render.PictOp.Src,
                    pic_gradient,
                    0,
                    pic_window,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    WIDTH,
                    HEIGHT,
                )

                img = conn.core.GetImage(
                    xcffib.xproto.ImageFormat.ZPixmap,
                    window,
                    0,
                    0,
                    WIDTH,
                    HEIGHT,
                    0xFFFFFFFF,
                ).reply()

                conn.flush()

                assert img.data[0:2] == [0, 0]
                assert img.data[-2:] == [255, 255]
                break



import xcffib
import struct
import io

MAJOR_VERSION = 4
MINOR_VERSION = 1
key = xcffib.ExtensionKey("XFree86-DRI")
_events = {}
_errors = {}


class DrmClipRect(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.x1, self.y1, self.x2, self.x3 = unpacker.unpack("=hhhh")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=hhhh", self.x1, self.y1, self.x2, self.x3))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, x1, y1, x2, x3):
        self = cls.__new__(cls)
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.x3 = x3
        return self


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.dri_major_version, self.dri_minor_version, self.dri_minor_patch = (
            unpacker.unpack("=xx2x4xHHI")
        )
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class QueryDirectRenderingCapableReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.is_capable,) = unpacker.unpack("=xx2x4xB")
        self.bufsize = unpacker.offset - base


class QueryDirectRenderingCapableCookie(xcffib.Cookie):
    reply_type = QueryDirectRenderingCapableReply


class OpenConnectionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.sarea_handle_low, self.sarea_handle_high, self.bus_id_len = (
            unpacker.unpack("=xx2x4xIII12x")
        )
        self.bus_id = xcffib.List(unpacker, "c", self.bus_id_len)
        self.bufsize = unpacker.offset - base


class OpenConnectionCookie(xcffib.Cookie):
    reply_type = OpenConnectionReply


class GetClientDriverNameReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.client_driver_major_version,
            self.client_driver_minor_version,
            self.client_driver_patch_version,
            self.client_driver_name_len,
        ) = unpacker.unpack("=xx2x4xIIII8x")
        self.client_driver_name = xcffib.List(
            unpacker, "c", self.client_driver_name_len
        )
        self.bufsize = unpacker.offset - base


class GetClientDriverNameCookie(xcffib.Cookie):
    reply_type = GetClientDriverNameReply


class CreateContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.hw_context,) = unpacker.unpack("=xx2x4xI")
        self.bufsize = unpacker.offset - base


class CreateContextCookie(xcffib.Cookie):
    reply_type = CreateContextReply


class CreateDrawableReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.hw_drawable_handle,) = unpacker.unpack("=xx2x4xI")
        self.bufsize = unpacker.offset - base


class CreateDrawableCookie(xcffib.Cookie):
    reply_type = CreateDrawableReply


class GetDrawableInfoReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.drawable_table_index,
            self.drawable_table_stamp,
            self.drawable_origin_X,
            self.drawable_origin_Y,
            self.drawable_size_W,
            self.drawable_size_H,
            self.num_clip_rects,
            self.back_x,
            self.back_y,
            self.num_back_clip_rects,
        ) = unpacker.unpack("=xx2x4xIIhhhhIhhI")
        self.clip_rects = xcffib.List(unpacker, DrmClipRect, self.num_clip_rects)
        unpacker.pad(DrmClipRect)
        self.back_clip_rects = xcffib.List(
            unpacker, DrmClipRect, self.num_back_clip_rects
        )
        self.bufsize = unpacker.offset - base


class GetDrawableInfoCookie(xcffib.Cookie):
    reply_type = GetDrawableInfoReply


class GetDeviceInfoReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.framebuffer_handle_low,
            self.framebuffer_handle_high,
            self.framebuffer_origin_offset,
            self.framebuffer_size,
            self.framebuffer_stride,
            self.device_private_size,
        ) = unpacker.unpack("=xx2x4xIIIIII")
        self.device_private = xcffib.List(unpacker, "I", self.device_private_size)
        self.bufsize = unpacker.offset - base


class GetDeviceInfoCookie(xcffib.Cookie):
    reply_type = GetDeviceInfoReply


class AuthConnectionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.authenticated,) = unpacker.unpack("=xx2x4xI")
        self.bufsize = unpacker.offset - base


class AuthConnectionCookie(xcffib.Cookie):
    reply_type = AuthConnectionReply


class xf86driExtension(xcffib.Extension):
    def QueryVersion(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self):
        return self.QueryVersion(is_checked=True)

    def QueryVersionUnchecked(self):
        return self.QueryVersion(is_checked=False)

    def QueryDirectRenderingCapable(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", screen))
        return self.send_request(
            1, buf, QueryDirectRenderingCapableCookie, is_checked=is_checked
        )

    def QueryDirectRenderingCapableChecked(self, screen):
        return self.QueryDirectRenderingCapable(screen, is_checked=True)

    def QueryDirectRenderingCapableUnchecked(self, screen):
        return self.QueryDirectRenderingCapable(screen, is_checked=False)

    def OpenConnection(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", screen))
        return self.send_request(2, buf, OpenConnectionCookie, is_checked=is_checked)

    def OpenConnectionChecked(self, screen):
        return self.OpenConnection(screen, is_checked=True)

    def OpenConnectionUnchecked(self, screen):
        return self.OpenConnection(screen, is_checked=False)

    def CloseConnection(self, screen, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", screen))
        return self.send_request(3, buf, is_checked=is_checked)

    def CloseConnectionChecked(self, screen):
        return self.CloseConnection(screen, is_checked=True)

    def CloseConnectionUnchecked(self, screen):
        return self.CloseConnection(screen, is_checked=False)

    def GetClientDriverName(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", screen))
        return self.send_request(
            4, buf, GetClientDriverNameCookie, is_checked=is_checked
        )

    def GetClientDriverNameChecked(self, screen):
        return self.GetClientDriverName(screen, is_checked=True)

    def GetClientDriverNameUnchecked(self, screen):
        return self.GetClientDriverName(screen, is_checked=False)

    def CreateContext(self, screen, visual, context, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIII", screen, visual, context))
        return self.send_request(5, buf, CreateContextCookie, is_checked=is_checked)

    def CreateContextChecked(self, screen, visual, context):
        return self.CreateContext(screen, visual, context, is_checked=True)

    def CreateContextUnchecked(self, screen, visual, context):
        return self.CreateContext(screen, visual, context, is_checked=False)

    def DestroyContext(self, screen, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", screen, context))
        return self.send_request(6, buf, is_checked=is_checked)

    def DestroyContextChecked(self, screen, context):
        return self.DestroyContext(screen, context, is_checked=True)

    def DestroyContextUnchecked(self, screen, context):
        return self.DestroyContext(screen, context, is_checked=False)

    def CreateDrawable(self, screen, drawable, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", screen, drawable))
        return self.send_request(7, buf, CreateDrawableCookie, is_checked=is_checked)

    def CreateDrawableChecked(self, screen, drawable):
        return self.CreateDrawable(screen, drawable, is_checked=True)

    def CreateDrawableUnchecked(self, screen, drawable):
        return self.CreateDrawable(screen, drawable, is_checked=False)

    def DestroyDrawable(self, screen, drawable, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", screen, drawable))
        return self.send_request(8, buf, is_checked=is_checked)

    def DestroyDrawableChecked(self, screen, drawable):
        return self.DestroyDrawable(screen, drawable, is_checked=True)

    def DestroyDrawableUnchecked(self, screen, drawable):
        return self.DestroyDrawable(screen, drawable, is_checked=False)

    def GetDrawableInfo(self, screen, drawable, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", screen, drawable))
        return self.send_request(9, buf, GetDrawableInfoCookie, is_checked=is_checked)

    def GetDrawableInfoChecked(self, screen, drawable):
        return self.GetDrawableInfo(screen, drawable, is_checked=True)

    def GetDrawableInfoUnchecked(self, screen, drawable):
        return self.GetDrawableInfo(screen, drawable, is_checked=False)

    def GetDeviceInfo(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", screen))
        return self.send_request(10, buf, GetDeviceInfoCookie, is_checked=is_checked)

    def GetDeviceInfoChecked(self, screen):
        return self.GetDeviceInfo(screen, is_checked=True)

    def GetDeviceInfoUnchecked(self, screen):
        return self.GetDeviceInfo(screen, is_checked=False)

    def AuthConnection(self, screen, magic, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", screen, magic))
        return self.send_request(11, buf, AuthConnectionCookie, is_checked=is_checked)

    def AuthConnectionChecked(self, screen, magic):
        return self.AuthConnection(screen, magic, is_checked=True)

    def AuthConnectionUnchecked(self, screen, magic):
        return self.AuthConnection(screen, magic, is_checked=False)


xcffib._add_ext(key, xf86driExtension, _events, _errors)
